//
//  AppDelegate.swift
//  DigitalInput
//
//  Created by Phidgets on 2018-02-13.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    let ch = DigitalInput()
    @IBOutlet weak var phidgetInfoBox: PhidgetInfoBox!
    
    //GUI
    @IBOutlet weak var powerSupplyPopUp: NSPopUpButton!
    @IBOutlet weak var inputModePopUp: NSPopUpButton!
    @IBOutlet weak var settingsBox: NSBox!
    @IBOutlet weak var stateLabel: NSTextField!
    @IBOutlet weak var stateBox: NSBox!
    @IBOutlet weak var dataBox: NSBox!
    

    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0

    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.stateChange.addHandler(statechange_handler)
            
            /*
             * Please review the Phidget22 channel matching documentation for details on the device
             * and class architecture of Phidget22, and how channels are matched to device features.
             */
            
            /*
             * Specifies the serial number of the device to attach to.
             * For VINT devices, this is the hub serial number.
             *
             * The default is any device.
             */
            // try ch.setDeviceSerialNumber(<YOUR DEVICE SERIAL NUMBER>)
            
            /*
             * For VINT devices, this specifies the port the VINT device must be plugged into.
             *
             * The default is any port.
             */
            // try ch.setHubPort(0)
            
            /*
             * Specifies which channel to attach to.  It is important that the channel of
             * the device is the same class as the channel that is being opened.
             *
             * The default is any channel.
             */
            // try ch.setChannel(0)
            
            /*
             * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
             * In a normal environment this can be done automatically by enabling server discovery, which
             * will cause the client to discovery and connect to available servers.
             *
             * To force the channel to only match a network Phidget, set remote to 1.
             */
            
            // try Net.enableServerDiscovery(serverType: .deviceRemote)
            // try ch.setIsRemote(true)
            
            /*
             openCmdLine allows command line arguments to be passed to this program in order to
             configure channel matching.
             If you have modified the code and added your own channel matching, comment out
             the openCmdLine call, and simply replace it with the following line:
             */
            //try ch.open()
            
            try phidgetInfoBox.openCmdLine(phid: ch as Phidget)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            ch.error.removeAllHandlers()
            ch.stateChange.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Phidget){
        do{
            let attachedDevice = sender as! DigitalInput
            let deviceID = try attachedDevice.getDeviceID()
            var powerSupply:PowerSupply?
            var inputMode:InputMode?
            
            if(deviceID == DeviceID.PN_DAQ1400){
                powerSupply = try attachedDevice.getPowerSupply()
                inputMode = try attachedDevice.getInputMode()
            }
            
            DispatchQueue.main.async {
                self.phidgetInfoBox.fillPhidgetInfo(device: sender)
                self.dataBox.isHidden = false
                
                self.stateLabel.stringValue = "?"
                self.stateLabel.textColor = NSColor.black
                self.stateBox.fillColor = NSColor.white
                
                if(deviceID == DeviceID.PN_DAQ1400){
                    self.settingsBox.isHidden = false
                    
                    self.powerSupplyPopUp.addItem(withTitle: "Off")
                    self.powerSupplyPopUp.addItem(withTitle: "12V")
                    self.powerSupplyPopUp.addItem(withTitle: "24V")
                    self.powerSupplyPopUp.selectItem(at: powerSupply == PowerSupply.off ? 0 : powerSupply == PowerSupply.volts_12 ? 1 : 2)
                    
                    self.inputModePopUp.addItem(withTitle: "NPN")
                    self.inputModePopUp.addItem(withTitle: "PNP")
                    self.inputModePopUp.selectItem(at: inputMode == InputMode.NPN ? 0 : 1)
                }
                
                //Adjusting window height
                var frame = self.window.frame
                frame.size.height = self.window.maxSize.height
                self.window.setFrame(frame, display: true, animate: false)
                self.window.center()
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: nil)
            self.settingsBox.isHidden = true
            self.dataBox.isHidden = true
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        outputError(errorDescription: data.description, errorCode: data.code.rawValue)
    }
    
    func statechange_handler(sender: DigitalInput, state: Bool){
        DispatchQueue.main.async {
            self.stateLabel.stringValue = state ? "T" : "F"
            self.stateLabel.textColor = state ? NSColor.black : NSColor.white
            self.stateBox.fillColor = state ? NSColor.white : NSColor.black
            print("State: ", state, "\n")
            
        }
    }
    
    //MARK: GUI Controls
    @IBAction func setPowerSupply(_ sender: Any) {
        do{
            let popup = sender as! NSPopUpButton
            if(popup.indexOfSelectedItem == 0){
                try ch.setPowerSupply(PowerSupply.off)
            }
            else if(popup.indexOfSelectedItem == 1){
                try ch.setPowerSupply(PowerSupply.volts_12)
            }
            else{
                try ch.setPowerSupply(PowerSupply.volts_24)
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setInputMode(_ sender: Any) {
        do{
            let popup = sender as! NSPopUpButton
            if(popup.indexOfSelectedItem == 0){
                try ch.setInputMode(InputMode.NPN)
            }
            else{
                try ch.setInputMode(InputMode.PNP)
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
        
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
    
}

